package logic

import (
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/file"
	"fusenapi/utils/hash"
	"time"

	"context"

	"fusenapi/server/upload/internal/svc"
	"fusenapi/server/upload/internal/types"

	"github.com/aws/aws-sdk-go/aws"
	"github.com/aws/aws-sdk-go/aws/request"
	"github.com/aws/aws-sdk-go/service/s3"
	"github.com/zeromicro/go-zero/core/logx"
)

type UploadFileBaseLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewUploadFileBaseLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UploadFileBaseLogic {
	return &UploadFileBaseLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *UploadFileBaseLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *UploadFileBaseLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }

func (l *UploadFileBaseLogic) UploadFileBase(req *types.UploadFileBaseReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null
	// 定义用户ID和S3键名格式
	var userId int64
	var guestId int64

	// 检查用户是否是游客
	if userinfo != nil {
		if userinfo.IsGuest() {
			// 如果是，使用游客ID和游客键名格式
			guestId = userinfo.GuestId
		} else {
			// 否则，使用用户ID和用户键名格式
			userId = userinfo.UserId
		}
		if guestId == 0 {
			guestId = req.GuestId
		}
		if userId == 0 {
			userId = req.UserId
		}
	}

	// 定义存储桶名称
	var bucketName *string
	var apiType int64 = req.ApiType

	// 根据类别选择存储桶
	switch req.UploadBucket {
	case 2:
		bucketName = basic.TempfileBucketName
	default:
		bucketName = basic.StorageBucketName
	}

	// 设置AWS会话的区域
	l.svcCtx.AwsSession.Config.Region = aws.String("us-west-1")

	// 创建新的S3服务实例
	svc := s3.New(l.svcCtx.AwsSession)

	// 定义S3请求和当前时间
	var s3req *request.Request

	var resourceId string = hash.JsonHashKey(req.FileKey)

	var uploadUrl = UploadUrl{}
	resourceModel := gmodel.NewFsResourceModel(l.svcCtx.MysqlConn)
	resourceInfo, err := resourceModel.FindOneById(l.ctx, resourceId)
	if err == nil && resourceInfo.ResourceId != "" {
		uploadUrl.Status = 1
		uploadUrl.ResourceId = resourceId
		uploadUrl.ResourceUrl = *resourceInfo.ResourceUrl
	} else {
		dist, contentType, err := file.FileBase64ToByte(req.FileData)

		if err != nil {
			logx.Error(err)
			return resp.SetStatus(basic.CodeFileUploadErr, "file upload err,base64tobyte error")
		}

		// 创建S3对象存储请求
		s3req, _ = svc.PutObjectRequest(
			&s3.PutObjectInput{
				Bucket: bucketName,
				Key:    &resourceId,
			},
		)

		// 设置请求体为文件数据
		s3req.SetBufferBody(dist)

		// 发送请求
		err = s3req.Send()

		// 检查是否有错误
		if err != nil {
			logx.Error(err)
			uploadUrl.Status = 0
		} else {
			var url = s3req.HTTPRequest.URL.String()
			// 打印请求URL
			logx.Info(url)
			uploadUrl.Status = 1
			uploadUrl.ResourceId = resourceId
			uploadUrl.ResourceUrl = url
			var version string = "0.0.1"
			var nowTime = time.Now()
			_, err = resourceModel.Create(l.ctx, &gmodel.FsResource{
				ResourceId:   resourceId,
				UserId:       &userId,
				GuestId:      &guestId,
				ResourceType: &contentType,
				ResourceUrl:  &url,
				Version:      &version,
				UploadedAt:   &nowTime,
				Metadata:     &req.Metadata,
				ApiType:      &apiType,
				BucketName:   bucketName,
			})
			if err != nil {
				logx.Error(err)
			}
		}
	}

	// 返回成功的响应和上传URL
	return resp.SetStatus(basic.CodeOK, map[string]interface{}{
		"upload_data": uploadUrl,
	})
}
